#!/usr/bin/env python
# coding: utf-8
#
# This module provides a not fully implemented gtk.StatusIcon
# based on egg.trayicon for stable systems pygtk < 2.10 
# 
# USAGE: Copy this file into the directory containing your python
# application and include these lines in your module:
#
# import gtk
# if not hasattr(gtk, "StatusIcon"):
#    import gtkx
#    gtk.__dict__["StatusIcon"]=gtkx.StatusIcon
#
#
# Copyright (C) Big-Cat Software Association, 2007.
#
# GILDE project.
# (Graphical Interfaces to Learn Debian Easily)
#
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import os
import sys
import gtk
import egg.trayicon
try:
    from sugar.activity import activity
    HAVE_SUGAR = True
except:
    HAVE_SUGAR = False

def gtkx():
# class
    gtk.__dict__["StatusIcon"]=StatusIcon
# functions
    gtk.__dict__["status_icon_new_from_pixbuf"]=status_icon_new_from_pixbuf
    gtk.__dict__["status_icon_new_from_file"]=status_icon_new_from_file
    gtk.__dict__["status_icon_new_from_stock"]=status_icon_new_from_stock
    gtk.__dict__["status_icon_new_from_icon_name"]=status_icon_new_from_icon_name
    gtk.__dict__["status_icon_position_menu"]=status_icon_position_menu


class StatusIcon:
    def __init__(self):

        self.trayeventbox = gtk.EventBox()
        self.trayicon = egg.trayicon.TrayIcon("TrayIcon")
        self.iconfactory = gtk.IconFactory()
#        self.InitStatusIcon()

#    def InitStatusIcon(self):
        self.trayimage = gtk.Image()
        self.trayeventbox.add(self.trayimage)
        self.trayicon.add(self.trayeventbox)
        self.tips = gtk.Tooltips()
        self.blinking=False

    def set_tooltip(self, tool_tip):
        self.tips.set_tip(self.trayicon, tool_tip)
        self.tips.enable()


    def new_icon(self, icon_name, file=None, fullpath=None):
        # Either the file or fullpath must be supplied, but not both:
        iconset = gtk.IconSet()
        if file:
            print "Adding icon file has not been implemented yet"
            #filename = [self.find_path(file)]
        else:
            filename = [fullpath]
        icons = [gtk.IconSource() for i in filename]
        for i, iconsource in enumerate(icons):
            iconsource.set_filename(filename[i])
            iconset.add_source(iconsource)
        self.iconfactory.add(icon_name, iconset)
        self.iconfactory.add_default()


    def set_from_pixbuf(self, pixbuf):
        #pixbuf : a gtk.gdk.Pixbuf or None 
        #Makes status_icon display pixbuf.
        self.pixbuf=pixbuf
        self.trayimage.set_from_pixbuf(self.pixbuf)


    def set_from_file(self, filename):
        self.file = self.find_path(filename)
        if not self.iconfactory.lookup(self.file):
            self.new_icon(self.file, fullpath=self.file)

        self.trayimage.set_from_stock(self.file, gtk.ICON_SIZE_BUTTON)


    def find_path(self, filename):
        full_filename = None
        if HAVE_SUGAR:
            full_filename = os.path.join(activity.get_bundle_path(), 'share', filename)
        else:
            if os.path.exists(os.path.join(sys.prefix, 'share', 'pixmaps', filename)):
                full_filename = os.path.join(sys.prefix, 'share', 'pixmaps', filename)
            elif os.path.exists(os.path.join(os.path.split(__file__)[0], filename)):
                full_filename = os.path.join(os.path.split(__file__)[0], filename)
            elif os.path.exists(os.path.join(os.path.split(__file__)[0], 'pixmaps', filename)):
                full_filename = os.path.join(os.path.split(__file__)[0], 'pixmaps', filename)
            elif os.path.exists(os.path.join(os.path.split(__file__)[0], 'share', filename)):
                full_filename = os.path.join(os.path.split(__file__)[0], 'share', filename)
            elif os.path.exists(os.path.join(__file__.split('/lib')[0], 'share', 'pixmaps', filename)):
                full_filename = os.path.join(__file__.split('/lib')[0], 'share', 'pixmaps', filename)
        if not full_filename:
            print filename + " cannot be found."
        return full_filename



    def set_from_stock(self, stock_id):
        #stock_id : a stock icon id
        #Makes status_icon display the stock icon with the id stock_id.
        self.stock=stock_id
        self.trayimage.set_from_stock(self.stock, gtk.ICON_SIZE_BUTTON)

    def set_from_icon_name(self, icon_name):
        #icon_name : an icon name 
        #Makes status_icon display the icon named icon_name from the current icon theme. 
        self.icon_name=icon_name
        self.trayimage.set_from_icon_name(self.icon_name, gtk.ICON_SIZE_BUTTON)

    def get_storage_type(self):
        #Returns : the image representation being used - one of the GTK Image Type Constants
        #Gets the type of representation being used by the gtk.StatusIcon to store image data.
        #gtk.IMAGE_EMPTY There is no image displayed by the widget 
        #gtk.IMAGE_PIXMAP The widget contains a gtk.gdk.Pixmap 
        #gtk.IMAGE_IMAGE The widget contains a gtk.gdk.Image 
        #gtk.IMAGE_PIXBUF The widget contains a gtk.gdk.Pixbuf 
        #gtk.IMAGE_STOCK The widget contains a stock icon name (see the Stock Items reference) 
        #gtk.IMAGE_ICON_SET The widget contains a gtk.IconSet 
        #gtk.IMAGE_ANIMATION The widget contains a gtk.gdk.PixbufAnimation
        return self.trayimage.get_storage_type()




    def get_pixbuf(self):
        #Returns : the displayed pixbuf, or None if the image is empty.
        #Gets the gtk.gdk.Pixbuf being displayed by the gtk.StatusIcon
        return self.trayimage.get_pixbuf()


    def get_stock(self):
        #Returns : stock id of the displayed stock icon, or None if the image is empty. 
        #Gets the id of the stock icon being displayed by the gtk.StatusIcon.
        return self.trayimage.get_stock()


    def get_icon_name(self):
        #Returns : name of the displayed icon, or None if the image is empty. 
        #Gets the name of the icon being displayed by the gtk.StatusIcon.
        return self.trayimage.get_icon_name()


    def get_size(self):
        #Returns :  the pixel size used for named icons.
        #The get_pixel_size() method returns the value of the "pixel-size" property which specifies the pixel size to be used for named icons.
        return self.trayimage.get_pixel_size()


    def connect(self, event_name, slot_name, sender=None):
        if event_name=='activate':
            self.trayeventbox.connect('button_press_event', self.activate)
            self.slot_activate=slot_name

        if event_name=='popup-menu':
            self.passed_menu=sender
            self.trayeventbox.connect('button_press_event', self.popup_menu)


    def activate(self, widget, event):
        if event.button==1:
            self.slot_activate(None)


    def popup_menu(self, widget, event):
        if event.button==3:
            self.passed_menu.show_all()
            self.passed_menu.popup(None, None, None, event.button, activate_time=event.time)


    def set_visible(self, show_flag):
        if show_flag:
            self.trayicon.show_all()
            self.visible=True
        if not show_flag:
            self.trayicon.hide_all()
            self.visible=False


    def get_visible(self):
        #Returns : True if the status icon is visible 
        #Returns True if the status icon is visible. Note that being visible does not guarantee that the user can actually see the icon, see also the gtk.StatusIcon.is_embedded() method.
        if self.trayicon.get_property("visible")==True:
            return True
        else:
            return False

# Signals
# user_param1, ...):

# The "activate" gtk.StatusIcon Signal
    def callback(self, status_icon, user_param1):
        #status_icon :  the object which received the signal
        #user_param1 :  the first user parameter (if any) specified with the connect() method
        #... :  additional user parameters (if any)
        #Gets emitted when the user activates the status icon. If and how status icons can be activated is platform-dependent.
        print "The 'activate' gtk.StatusIcon Signal has not been implemented yet"

# The "popup-menu" gtk.StatusIcon Signal
    def callback(self, status_icon, button, activate_time, user_param1):
        #status_icon :  the object which received the signal
        #button :  the button that was pressed, or 0 if the signal is not emitted in response to a button press event
        #activate_time :  the timestamp of the event that triggered the signal emission
        #user_param1 :  the first user parameter (if any) specified with the connect() method
        #... :  additional user parameters (if any)
        #Gets emitted when the user brings up the context menu of the status icon. Whether status icons can have context menus and how these are activated is platform-dependent.
        #The button and activate_timeout parameters should be passed as the last two arguments to the gtk.Menu.popup() method.
        print "The 'popup-menu' gtk.StatusIcon Signal has not been implemented yet"

# The "size-changed" gtk.StatusIcon Signal
    def callback(self, status_icon, size, statusicon, arg1, user_param1):
        #status_icon :  the object which received the signal
        #size :  the new size
        #user_param1 :  the first user parameter (if any) specified with the connect() method
        #... :  additional user parameters (if any)
        #Returns :  True if the icon was updated for the new size. Otherwise, GTK+ will scale the icon as necessary.
        #Gets emitted when the size available for the image changes, e.g. because the notification area got resized.
        print "The 'size-changed' gtk.StatusIcon Signal has not been implemented yet"


    def __getattr__(self, attr):
        print "WARNING: gtkx module does not contain all of the StatusIcon implementations"
        raise AttributeError("module has no attribute '%s'" % attr)



# Status icon from new: returns a new gtk.StatusIcon
# These are gtk. functions

def status_icon_new_from_pixbuf(pixbuf):
    #pixbuf :
    #Returns :  a new gtk.StatusIcon
    #Creates a status icon displaying pixbuf. The image will be scaled down to fit in the available space in the notification area, if necessary.
    Icon = StatusIcon()
    Icon.trayimage.set_from_pixbuf(pixbuf)
    return Icon

def status_icon_new_from_file(filename):
    #filename :
    #Returns :  a new gtk.StatusIcon
    #Creates a status icon displaying the image in the file specified by filename. The image will be scaled down to fit in the available space in the notification area, if necessary.
    Icon = StatusIcon()
    Icon.file = Icon.find_path(filename)
    if not Icon.iconfactory.lookup(Icon.file):
        Icon.new_icon(Icon.file, fullpath=Icon.file)
    Icon.trayimage.set_from_stock(Icon.file, gtk.ICON_SIZE_BUTTON)
    return Icon


def status_icon_new_from_stock(stock_id):
    #stock_id :  a stock icon id
    #Returns :  a new gtk.StatusIcon
    #Creates a status icon displaying a stock icon. Sample stock icon names are gtk.STOCK_OPEN, gtk.STOCK_QUIT. You can register your own stock icon names, see the gtk.IconFactory.add_default() and gtk.IconFactory.add() methods.
    Icon = StatusIcon()
    Icon.trayimage.set_from_stock(stock_id, gtk.ICON_SIZE_BUTTON)
    return Icon


def status_icon_new_from_icon_name(icon_name):
    #icon_name :  an icon name
    #Returns :  a new gtk.StatusIcon
    #Creates a status icon displaying an icon from the current icon theme. If the current icon theme is changed, the icon will be updated appropriately.
    Icon = StatusIcon()
    Icon.trayimage.set_from_icon_name(icon_name, gtk.ICON_SIZE_BUTTON)
    return Icon

def status_icon_position_menu(menu, statusicon):
    #menu :  the gtk.Menu
    #statusicon :  the gtk.StatusIcon
    #Returns :  a 3-tuple containing the x and y coordinates for the menu and a boolean indicating whether the menu should be pushed in to be completely inside the screen instead of just clamped to the size to the screen.
    #Menu positioning function to use with the popup() method to position menu aligned to statusicon .
    print "gtk.status_icon_position_menu has not been implemented yet"




